package lt05_pkg;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;

/**
 * This class encapsulates a polka dot shape, which has a diameter, a color, and
 * a position. It is also understood that a polka dot shape belongs to a
 * PolkaDotDataModel, and that the model should be notified anytime a polka
 * dot's state changes
 * 
 * @author mb
 * 
 */
public class PolkaDot implements Comparable<PolkaDot> {
	// public class PolkaDot {
	private Point topLeftAnchor;
	private Color theColor;
	private int diameter;
	private boolean isFilled = false;
	private double MOD_FACTOR = 0.1;

	private static Color[] PALETTE = { Color.RED, Color.ORANGE, Color.YELLOW,
			Color.GREEN, Color.BLUE, Color.MAGENTA };

	private PolkaDotDataModel dataModel;

	/**
	 * Construct a polka dot with the specified attributes.
	 * 
	 * @param xPos
	 *            the x-coordinate of the upper left hand anchor point of the
	 *            polka dot to be created.
	 * @param yPos
	 *            the y-coordinate of the upper left hand anchor point of the
	 *            polka dot to be created.
	 * @param col
	 *            the colour of the polka dot to be created.
	 * @param diameter
	 *            the diameter of the polka dot to be created.
	 * @param dataModel
	 *            the PolkaDotDataModel object to which this new polka dot
	 *            object will belong
	 */
	public PolkaDot(int xPos, int yPos, Color col, int diameter,
			PolkaDotDataModel dataModel) {
		this.topLeftAnchor = new Point(xPos, yPos);
		this.theColor = col;
		this.diameter = diameter;
		this.dataModel = dataModel;
	}

	/**
	 * Construct a polka dot with the specified attributes.
	 * 
	 * 
	 * @param point
	 *            the upper left hand anchor point of the polka dot to be
	 *            created.
	 * @param col
	 *            the colour of the polka dot to be created.
	 * @param diameter
	 *            the diameter of the polka dot to be created.
	 * @param dataModel
	 *            the PolkaDotDataModel object to which this new polka dot
	 *            object will belong
	 */
	public PolkaDot(Point point, Color col, int diameter,
			PolkaDotDataModel dataModel) {
		this(point.x, point.y, col, diameter, dataModel);
	}

	/**
	 * Specify the color of this Polka Dot and notify that model has changed
	 * (only if new color is different than current color).
	 * 
	 * @param newColor
	 */
	public void setTheColor(Color newColor) {
		if (!isSameColor(this.getTheColor(), newColor)) {
			this.theColor = newColor;
			dataModel.notifyModelHasChanged();
		}
	}

	/**
	 * Method that increases the size of this dot by 10%, while maintaining
	 * position of centre. Notify that the model has changed
	 */
	public void increaseSize() {
		int trans = (int) ((this.diameter * (MOD_FACTOR)) / 2);
		changeSize(1 + MOD_FACTOR);
		this.topLeftAnchor.translate(-trans, -trans);
		dataModel.notifyModelHasChanged();
	}

	/**
	 * Method that decreases the size of this dot by 10%, while maintaining
	 * position of centre. Notify that the model has changed
	 */
	public void decreaseSize() {
		int trans = (int) ((this.diameter * (MOD_FACTOR)) / 2);
		changeSize(1 - MOD_FACTOR);
		this.topLeftAnchor.translate(trans, trans);
		dataModel.notifyModelHasChanged();
	}

	private void changeSize(double factor) {
		this.diameter = (int) (diameter * factor);

	}

	private static boolean isSameColor(Color col1, Color col2) {
		return col1.getRGB() == col2.getRGB();
	}

	/**
	 * Get the color of this Polka Dot
	 * 
	 */
	public Color getTheColor() {
		return this.theColor;
	}

	/**
	 * This method causes this polka dot to be drawn on the passed Graphics2D
	 * objects.
	 * 
	 * 
	 * @param graphics2D
	 */
	public void drawComponent(Graphics2D graphics2D) {
		graphics2D.setStroke(new BasicStroke(4));
		graphics2D.setColor(this.getTheColor());
		if (isFilled) {
			graphics2D.fill(getTheShape());
		} else {
			graphics2D.draw(getTheShape());
		}
	}

	private Shape getTheShape() {
		return new Ellipse2D.Double(topLeftAnchor.x, topLeftAnchor.y, diameter,
				diameter);
	}

	/**
	 * This method generates a random polka dot, with a color randomly chosen
	 * from within a palette of 6 basic colors (red, orange, yellow, green,
	 * blue, magenta). The position is chosen randomly, but subject to the
	 * constraint that the polka dot is drawable within the size of the passed
	 * canvas. The diameter is chosen randomly from the interval [20,50]
	 * 
	 * @param xDim
	 * @param yDim
	 * @param dataModel
	 * @return a new polka dot object, as specified above
	 */
	public static PolkaDot getRandom(int xDim, int yDim,
			PolkaDotDataModel dataModel) {
		final int MIN_DIA = 20;
		final int MAX_DIA = 50;
		int diameter = PolkaDot
				.getRandomIntFromClosedInterval(MIN_DIA, MAX_DIA);
		int xPos = PolkaDot.getRandomIntFromClosedInterval(0, xDim - diameter);
		int yPos = PolkaDot.getRandomIntFromClosedInterval(0, yDim - diameter);
		return new PolkaDot(xPos, yPos, getRandomColor(), diameter, dataModel);
	}

	/**
	 * Returns a color randomly chosen from within a palette of 6 basic colors
	 * (red, orange, yellow, green, blue, magenta).
	 * 
	 * @return a Color, as specified above
	 */
	public static Color getRandomColor() {
		int colIdx = PolkaDot.getRandomIntFromClosedInterval(0,
				PALETTE.length - 1);
		return PALETTE[colIdx];
	}

	private static int getRandomIntFromClosedInterval(int minDesiredVal,
			int maxDesiredVal) {
		int randomVal = minDesiredVal
				+ (int) (Math.random() * (maxDesiredVal - minDesiredVal + 1));
		return randomVal;
	}

	public Point getLocationOfTopLeftAnchor() {
		return topLeftAnchor;
	}

	public Point getCentrePoint() {
		Point p = new Point(topLeftAnchor);
		p.translate(diameter / 2, diameter / 2);
		return p;
	}

	@Override
	public String toString() {
		return "Polka Dot, pos : " + this.getLocationOfTopLeftAnchor();
	}

	/**
	 * This method determines whether the passed point is contained within this
	 * polka dot
	 * 
	 * @param p
	 * @return the boolean value as specifed above
	 */
	public boolean contains(Point p) {
		return getTheShape().contains(p);
	}

	/**
	 * Method that determines whether the passed dot is a duplicate of this dot.
	 * A duplicate is any polka dot with the same centre point (even if the
	 * diameters are different)
	 * 
	 * @param dot
	 * @return a boolean value, as specified above
	 */
	public boolean isDuplicate(PolkaDot dot) {
		// System.out.println(this.getCentrePoint().x + " "
		// + dot.getCentrePoint().x + " " + this.getCentrePoint().y + " "
		// + dot.getCentrePoint().y);
		return this.getCentrePoint().x == dot.getCentrePoint().x
				&& this.getCentrePoint().y == dot.getCentrePoint().y;
	}

	/**
	 * Method that indicates whether this polka dot is a shape that is meant to
	 * be filled or unfilled
	 * 
	 * @return a boolean value as specified above
	 */
	public boolean isFilled() {
		return isFilled;
	}

	/**
	 * Method to specify whether this polka dot should be a filled or unfilled
	 * shape.
	 * 
	 * @param isFilled
	 */
	public void setFilled(boolean isFilled) {
		this.isFilled = isFilled;
	}

	@Override
	public int compareTo(PolkaDot otherDot) {
		System.out.println("in compareTo(PolkaDot)");
		return this.getTheColor().getRGB() - otherDot.getTheColor().getRGB();
	}

	@Override
	public boolean equals(Object object) {
		System.out.println("in .equals(Object)");
		if (object == null)
			return false;
		if (object == this)
			return true;
		if (this.getClass() != object.getClass())
			return false;
		else {
			// here we are certain that the cast is safe
			return isDuplicate((PolkaDot) object);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		// The proper semantics are that objects that are equal should have the
		// same hashCode. Objects that are not equal should be as likely as
		// possible to have a different hashCode. This is not a requirement, but
		// will improve performance. If two objects are found to have different
		// hashCodes, then
		// equals will be invoked
		System.out.println("in .hashCode()");
		StringBuffer buffer = new StringBuffer();
		buffer.append(this.getCentrePoint().toString());
		return buffer.toString().hashCode();
	}

}
