package lt05_pkg;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Vector;

/**
 * This class encapsulates a data model of a collection of polka dots.
 * 
 * Duplicate polka dots are permitted.
 * 
 * @author mb
 * 
 */
public class PolkaDotDataModel {

	private Set<PolkaDot> theDots;
	private List<ModelListener> listeners;

	/**
	 * Creates a collection and initialized it with three polka dots
	 */
	public PolkaDotDataModel() {
		listeners = new Vector<ModelListener>();
		theDots = new HashSet<PolkaDot>();
		// here we add a few polka dots to populate the model with some initial
		// data
		this.addDot(new PolkaDot(30, 40, Color.RED, 30, this));
		this.addDot(new PolkaDot(30, 40, Color.BLUE, 30, this));
		this.addDot(new PolkaDot(50, 20, Color.MAGENTA, 20, this));
		this.addDot(new PolkaDot(100, 80, Color.GREEN, 50, this));
	}

	/**
	 * Add a listener to this model.
	 * 
	 * @param listener
	 */
	public void addListener(ModelListener listener) {
		listeners.add(listener);
	}

	public void notifyModelHasChanged() {
		for (ModelListener listener : listeners) {
			listener.changed();
		}
	}

	/**
	 * Return this collection as a List of PolkaDot objects.
	 * 
	 * @return as described above
	 */
	public Set<PolkaDot> getDots() {
		return theDots;
	}

	/**
	 * Add the passed polka dot to this collection. The model notifies its
	 * listeners (if any) that it has changed. As well, a statement is printed
	 * to the console "Model is changing." This method will not permit a
	 * duplicate polka dot to be added. See the PolkaDot's method equals() to
	 * see what constitutes a duplicate.
	 * 
	 * @param theDot
	 *            a non-null Polka Dot object
	 * @return whether the dot was added or not
	 */
	public boolean addDot(PolkaDot theDot) {
		boolean wasAdded = theDots.add(theDot);
		System.out.println("was added " + wasAdded);
		if (wasAdded)
			System.out.println("Model is changing.");
		notifyModelHasChanged();
		return wasAdded;
	}

	/**
	 * This method returns the polka dot in this model that is closest to the
	 * passed point. If this model has no polka dots, this method will return
	 * null.
	 * 
	 * @param p
	 * @return as specified above
	 */
	public PolkaDot getClosestShape(Point p) {
		Double currDist = null;
		PolkaDot closest = null;
		for (PolkaDot pd : getDots()) {
			double dist = getDistance(pd.getCentrePoint(), p);
			if (closest == null || dist < currDist) {
				closest = pd;
				currDist = dist;
			}
		}	
		return closest;
	}

	/**
	 * This method returns the polka dot in this model that the passed point
	 * hits. If the passed point does not intersect with any polka dot in this
	 * model, then this method returns null. If the passed point intersects with
	 * multiple polka dots, then the polka dot with the centre point that is
	 * closest to the passed point is selected.
	 * 
	 * @param p
	 * @return the polka dot that is hit, according to the specification above,
	 *         or null if no polka dot is hit
	 */
	public PolkaDot getMatchingDot(Point p) {
		Double currDist = null;
		PolkaDot closest = null;
		for (PolkaDot pd : getDots()) {
			double dist = getDistance(pd.getCentrePoint(), p);
			if (closest == null || dist < currDist) {
				closest = pd;
				currDist = dist;
			}
		}
		if (closest.contains(p))
			return closest;
		else
			return null;
	}

	private double getDistance(Point location, Point p) {
		int distX = Math.abs(location.x - p.x);
		int distY = Math.abs(location.y - p.y);
		double dist = Math.sqrt(distX * distX + distY * distY);
		return dist;
	}

	public void setTheColor(PolkaDot p, Color c) {
		p.setTheColor(c);
		notifyModelHasChanged();
	}

}
